@@ -21,11 +21,12 @@ from oslo_log import log
 from oslo_utils import timeutils
 import six
 
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _, _LW
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -214,7 +215,7 @@ class DirectMaps(object):
         self._matches = []
 
     def add(self, values):
-        """Adds a matched value to the list of matches.
+        """Add a matched value to the list of matches.
 
         :param list value: the match to save
 
@@ -246,19 +247,6 @@ def validate_expiration(token_ref):
         raise exception.Unauthorized(_('Federation token is expired'))
 
 
-def validate_groups_cardinality(group_ids, mapping_id):
-    """Check if groups list is non-empty.
-
-    :param group_ids: list of group ids
-    :type group_ids: list of str
-
-    :raises keystone.exception.MissingGroups: if ``group_ids`` cardinality is 0
-
-    """
-    if not group_ids:
-        raise exception.MissingGroups(mapping_id=mapping_id)
-
-
 def get_remote_id_parameter(protocol):
     # NOTE(marco-fargetta): Since we support any protocol ID, we attempt to
     # retrieve the remote_id_attribute of the protocol ID. If it's not
@@ -268,6 +256,7 @@ def get_remote_id_parameter(protocol):
     try:
         remote_id_parameter = CONF[protocol]['remote_id_attribute']
     except AttributeError:
+        # TODO(dolph): Move configuration registration to keystone.conf
         CONF.register_opt(cfg.StrOpt('remote_id_attribute'),
                           group=protocol)
         try:
@@ -304,7 +293,7 @@ def validate_idp(idp, protocol, assertion):
         raise exception.Forbidden(msg)
 
 
-def validate_groups_in_backend(group_ids, mapping_id, identity_api):
+def validate_mapped_group_ids(group_ids, mapping_id, identity_api):
     """Iterate over group ids and make sure they are present in the backend.
 
     This call is not transactional.
@@ -330,35 +319,11 @@ def validate_groups_in_backend(group_ids, mapping_id, identity_api):
                 group_id=group_id, mapping_id=mapping_id)
 
 
-def validate_groups(group_ids, mapping_id, identity_api):
-    """Check group ids cardinality and check their existence in the backend.
-
-    This call is not transactional.
-    :param group_ids: IDs of the groups to be checked
-    :type group_ids: list of str
-
-    :param mapping_id: id of the mapping used for this operation
-    :type mapping_id: str
-
-    :param identity_api: Identity Manager object used for communication with
-                         backend
-    :type identity_api: identity.Manager
-
-    :raises keystone.exception.MappedGroupNotFound: If the group returned by
-        mapping was not found in the backend.
-    :raises keystone.exception.MissingGroups: If ``group_ids`` cardinality
-        is 0.
-
-    """
-    validate_groups_cardinality(group_ids, mapping_id)
-    validate_groups_in_backend(group_ids, mapping_id, identity_api)
-
-
 # TODO(marek-denis): Optimize this function, so the number of calls to the
 # backend are minimized.
 def transform_to_group_ids(group_names, mapping_id,
                            identity_api, resource_api):
-    """Transform groups identified by name/domain to their ids
+    """Transform groups identified by name/domain to their ids.
 
     Function accepts list of groups identified by a name and domain giving
     a list of group ids in return.
@@ -418,14 +383,14 @@ def transform_to_group_ids(group_names, mapping_id,
                 group['name'], resolve_domain(group['domain']))
             yield group_dict['id']
         except exception.GroupNotFound:
-            LOG.debug('Skip mapping group %s; has no entry in the backend',
-                      group['name'])
+            raise exception.MappedGroupNotFound(group_id=group['name'],
+                                                mapping_id=mapping_id)
 
 
-def get_assertion_params_from_env(context):
-    LOG.debug('Environment variables: %s', context['environment'])
+def get_assertion_params_from_env(request):
+    LOG.debug('Environment variables: %s', request.environ)
     prefix = CONF.federation.assertion_prefix
-    for k, v in list(context['environment'].items()):
+    for k, v in list(request.environ.items()):
         if not k.startswith(prefix):
             continue
         # These bytes may be decodable as ISO-8859-1 according to Section
