@@ -17,21 +17,27 @@ import uuid
 
 import fixtures
 import mock
-from oslo_config import cfg
+from oslo_config import fixture as config_fixture
+from oslo_log import log
+from oslotest import mockpatch
 from six.moves import range
 from testtools import matchers
 
 from keystone.auth import controllers
 from keystone.cmd import cli
 from keystone.common import dependency
+from keystone.common.sql import migration_helpers
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _
-from keystone import resource
+from keystone.identity.mapping_backends import mapping as identity_mapping
 from keystone.tests import unit
+from keystone.tests.unit import default_fixtures
 from keystone.tests.unit.ksfixtures import database
+from keystone.tests.unit.ksfixtures import ldapdb
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 class CliTestCase(unit.SQLDriverOverrides, unit.TestCase):
@@ -45,6 +51,43 @@ class CliTestCase(unit.SQLDriverOverrides, unit.TestCase):
         self.load_backends()
         cli.TokenFlush.main()
 
+    # NOTE(ravelar): the following method tests that the token_flush command,
+    # when used in conjunction with an unsupported token driver like kvs,
+    # will yield a LOG.warning message informing the user that the
+    # command had no effect.
+    def test_token_flush_excepts_not_implemented_and_logs_warning(self):
+        self.useFixture(database.Database())
+        self.load_backends()
+        self.config_fixture.config(group='token', driver='memcache')
+        log_info = self.useFixture(fixtures.FakeLogger(level=log.WARN))
+        cli.TokenFlush.main()
+        self.assertIn("token_flush command had no effect", log_info.output)
+
+
+class CliNoConfigTestCase(unit.BaseTestCase):
+
+    def setUp(self):
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+        self.config_fixture.register_cli_opt(cli.command_opt)
+        self.useFixture(mockpatch.Patch(
+            'oslo_config.cfg.find_config_files', return_value=[]))
+        super(CliNoConfigTestCase, self).setUp()
+
+        # NOTE(crinkle): the command call doesn't have to actually work,
+        # that's what the other unit tests are for. So just mock it out.
+        class FakeConfCommand(object):
+            def __init__(self):
+                self.cmd_class = mock.Mock()
+        self.useFixture(mockpatch.PatchObject(
+            CONF, 'command', FakeConfCommand()))
+
+        self.logging = self.useFixture(fixtures.FakeLogger(level=log.WARN))
+
+    def test_cli(self):
+        expected_msg = 'Config file not found, using default configs.'
+        cli.main(argv=['keystone-manage', 'db_sync'])
+        self.assertThat(self.logging.output, matchers.Contains(expected_msg))
+
 
 class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
 
@@ -80,12 +123,12 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
             bootstrap.assignment_manager.get_roles_for_user_and_project(
                 user['id'],
                 project['id']))
-        self.assertIs(len(role_list), 1)
+        self.assertIs(1, len(role_list))
         self.assertEqual(role_list[0], role['id'])
         # NOTE(morganfainberg): Pass an empty context, it isn't used by
         # `authenticate` method.
         bootstrap.identity_manager.authenticate(
-            {},
+            self.make_request(),
             user['id'],
             bootstrap.password)
 
@@ -133,20 +176,15 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 }
             }
         }
-
-        context = {'environment': {}, 'query_string': {}}
         auth_response = v3_token_controller.authenticate_for_token(
-            context, v3_password_data)
-        token = auth_response.headers.get('X-Subject-Token')
+            self.make_request(), v3_password_data)
+        token = auth_response.headers['X-Subject-Token']
         self._do_test_bootstrap(bootstrap)
         # build validation request
-        context = {'is_admin': True,
-                   'subject_token_id': token,
-                   'token_id': token,
-                   'query_string': {},
-                   'environment': {}}
+        request = self.make_request(is_admin=True)
+        request.context_dict['subject_token_id'] = token
         # Make sure the token we authenticate for is still valid.
-        v3_token_controller.validate_token(context)
+        v3_token_controller.validate_token(request)
 
     def test_bootstrap_is_not_idempotent_when_password_does_change(self):
         # NOTE(lbragstad): Ensure bootstrap isn't idempotent when run with
@@ -168,18 +206,14 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 }
             }
         }
-        context = {'environment': {}, 'query_string': {}}
         auth_response = v3_token_controller.authenticate_for_token(
-            context, v3_password_data)
-        token = auth_response.headers.get('X-Subject-Token')
+            self.make_request(), v3_password_data)
+        token = auth_response.headers['X-Subject-Token']
         os.environ['OS_BOOTSTRAP_PASSWORD'] = uuid.uuid4().hex
         self._do_test_bootstrap(bootstrap)
         # build validation request
-        context = {'is_admin': True,
-                   'subject_token_id': token,
-                   'token_id': token,
-                   'query_string': {},
-                   'environment': {}}
+        request = self.make_request(is_admin=True)
+        request.context_dict['subject_token_id'] = token
         # Since the user account was recovered with a different password, we
         # shouldn't be able to validate this token. Bootstrap should have
         # persisted a revocation event because the user's password was updated.
@@ -188,7 +222,7 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
         self.assertRaises(
             exception.TokenNotFound,
             v3_token_controller.validate_token,
-            context
+            request
         )
 
     def test_bootstrap_recovers_user(self):
@@ -209,10 +243,23 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
 
         # Sanity check that the original password works again.
         bootstrap.identity_manager.authenticate(
-            {},
+            self.make_request(),
             user_id,
             bootstrap.password)
 
+    def test_bootstrap_creates_default_role(self):
+        bootstrap = cli.BootStrap()
+        try:
+            role = bootstrap.role_manager.get_role(CONF.member_role_id)
+            self.fail('Member Role is created and should not be.')
+        except exception.RoleNotFound:
+            pass
+
+        self._do_test_bootstrap(bootstrap)
+        role = bootstrap.role_manager.get_role(CONF.member_role_id)
+        self.assertEqual(role['name'], CONF.member_role_name)
+        self.assertEqual(role['id'], CONF.member_role_id)
+
 
 class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
 
@@ -346,15 +393,10 @@ class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
 
     def config_files(self):
         self.config_fixture.register_cli_opt(cli.command_opt)
-        self.addCleanup(self.cleanup)
         config_files = super(CliDomainConfigAllTestCase, self).config_files()
         config_files.append(unit.dirs.tests_conf('backend_sql.conf'))
         return config_files
 
-    def cleanup(self):
-        CONF.reset()
-        CONF.unregister_opt(cli.command_opt)
-
     def cleanup_domains(self):
         for domain in self.domains:
             if domain == 'domain_default':
@@ -384,8 +426,11 @@ class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
             domain = 'domain%s' % x
             self.domains[domain] = create_domain(
                 {'id': uuid.uuid4().hex, 'name': domain})
-        self.domains['domain_default'] = create_domain(
-            resource.calc_default_domain())
+        self.default_domain = unit.new_domain_ref(
+            description=u'The default domain',
+            id=CONF.identity.default_domain_id,
+            name=u'Default')
+        self.domains['domain_default'] = create_domain(self.default_domain)
 
     def test_config_upload(self):
         # The values below are the same as in the domain_configs_multi_ldap
@@ -475,12 +520,11 @@ class CliDomainConfigSingleDomainTestCase(CliDomainConfigAllTestCase):
         dependency.reset()
         with mock.patch('six.moves.builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
-            file_name = ('keystone.%s.conf' %
-                         resource.calc_default_domain()['name'])
+            file_name = ('keystone.%s.conf' % self.default_domain['name'])
             error_msg = _(
                 'Domain: %(domain)s already has a configuration defined - '
                 'ignoring file: %(file)s.') % {
-                    'domain': resource.calc_default_domain()['name'],
+                    'domain': self.default_domain['name'],
                     'file': os.path.join(CONF.identity.domain_config_dir,
                                          file_name)}
             mock_print.assert_has_calls([mock.call(error_msg)])
@@ -571,3 +615,125 @@ class TestDomainConfigFinder(unit.BaseTestCase):
         self.assertThat(
             self.logging.output,
             matchers.Contains(expected_msg_template % 'keystone.conf'))
+
+
+class CliDBSyncTestCase(unit.BaseTestCase):
+
+    class FakeConfCommand(object):
+        def __init__(self, parent):
+            self.extension = False
+            self.expand = parent.command_expand
+            self.migrate = parent.command_migrate
+            self.contract = parent.command_contract
+            self.version = None
+
+    def setUp(self):
+        super(CliDBSyncTestCase, self).setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+        self.config_fixture.register_cli_opt(cli.command_opt)
+        migration_helpers.offline_sync_database_to_version = mock.Mock()
+        migration_helpers.expand_schema = mock.Mock()
+        migration_helpers.migrate_data = mock.Mock()
+        migration_helpers.contract_schema = mock.Mock()
+        self.command_expand = False
+        self.command_migrate = False
+        self.command_contract = False
+
+    def _assert_correct_call(self, mocked_function):
+        for func in [migration_helpers.offline_sync_database_to_version,
+                     migration_helpers.expand_schema,
+                     migration_helpers.migrate_data,
+                     migration_helpers.contract_schema]:
+            if func == mocked_function:
+                self.assertTrue(func.called)
+            else:
+                self.assertFalse(func.called)
+
+    def test_db_sync(self):
+        self.useFixture(mockpatch.PatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        cli.DbSync.main()
+        self._assert_correct_call(
+            migration_helpers.offline_sync_database_to_version)
+
+    def test_db_sync_expand(self):
+        self.command_expand = True
+        self.useFixture(mockpatch.PatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        cli.DbSync.main()
+        self._assert_correct_call(migration_helpers.expand_schema)
+
+    def test_db_sync_migrate(self):
+        self.command_migrate = True
+        self.useFixture(mockpatch.PatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        cli.DbSync.main()
+        self._assert_correct_call(migration_helpers.migrate_data)
+
+    def test_db_sync_contract(self):
+        self.command_contract = True
+        self.useFixture(mockpatch.PatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        cli.DbSync.main()
+        self._assert_correct_call(migration_helpers.contract_schema)
+
+
+class TestMappingPopulate(unit.SQLDriverOverrides, unit.TestCase):
+
+    def setUp(self):
+        sqldb = self.useFixture(database.Database())
+        super(TestMappingPopulate, self).setUp()
+        self.ldapdb = self.useFixture(ldapdb.LDAPDatabase())
+        self.ldapdb.clear()
+
+        self.load_backends()
+
+        sqldb.recreate()
+        self.load_fixtures(default_fixtures)
+
+    def config_files(self):
+        self.config_fixture.register_cli_opt(cli.command_opt)
+        config_files = super(TestMappingPopulate, self).config_files()
+        config_files.append(unit.dirs.tests_conf('backend_ldap_sql.conf'))
+        return config_files
+
+    def config_overrides(self):
+        super(TestMappingPopulate, self).config_overrides()
+        self.config_fixture.config(group='identity', driver='ldap')
+        self.config_fixture.config(group='identity_mapping',
+                                   backward_compatible_ids=False)
+
+    def config(self, config_files):
+        CONF(args=['mapping_populate', '--domain-name', 'Default'],
+             project='keystone',
+             default_config_files=config_files)
+
+    def test_mapping_populate(self):
+        # mapping_populate should create id mappings. Test plan:
+        # 0. Purge mappings
+        # 1. Fetch user list directly via backend. It will not create any
+        #    mappings because it bypasses identity manager
+        # 2. Verify that users have no public_id yet
+        # 3. Execute mapping_populate. It should create id mappings
+        # 4. For the same users verify that they have public_id now
+        purge_filter = {}
+        self.id_mapping_api.purge_mappings(purge_filter)
+        hints = None
+        users = self.identity_api.driver.list_users(hints)
+        for user in users:
+            local_entity = {
+                'domain_id': CONF.identity.default_domain_id,
+                'local_id': user['id'],
+                'entity_type': identity_mapping.EntityType.USER}
+            self.assertIsNone(self.id_mapping_api.get_public_id(local_entity))
+
+        dependency.reset()  # backends are loaded again in the command handler
+        cli.MappingPopulate.main()
+
+        for user in users:
+            local_entity = {
+                'domain_id': CONF.identity.default_domain_id,
+                'local_id': user['id'],
+                'entity_type': identity_mapping.EntityType.USER}
+            self.assertIsNotNone(
+                self.id_mapping_api.get_public_id(local_entity))
