@@ -14,16 +14,22 @@
 
 import uuid
 
-from oslo_config import cfg
+import fixtures
 from oslo_config import fixture as config_fixture
+from oslo_log import log
 from oslo_serialization import jsonutils
 import six
 
 from keystone.common import wsgi
+import keystone.conf
 from keystone import exception
+from keystone.i18n import _LE
 from keystone.tests import unit
 
 
+CONF = keystone.conf.CONF
+
+
 class ExceptionTestCase(unit.BaseTestCase):
     def assertValidJsonRendering(self, e):
         resp = wsgi.render_exception(e)
@@ -107,7 +113,7 @@ class ExceptionTestCase(unit.BaseTestCase):
 
 
 class UnexpectedExceptionTestCase(ExceptionTestCase):
-    """Tests if internal info is exposed to the API user on UnexpectedError."""
+    """Test if internal info is exposed to the API user on UnexpectedError."""
 
     class SubClassExc(exception.UnexpectedError):
         debug_message_format = 'Debug Message: %(debug_info)s'
@@ -115,7 +121,7 @@ class UnexpectedExceptionTestCase(ExceptionTestCase):
     def setUp(self):
         super(UnexpectedExceptionTestCase, self).setUp()
         self.exc_str = uuid.uuid4().hex
-        self.config_fixture = self.useFixture(config_fixture.Config(cfg.CONF))
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
 
     def test_unexpected_error_no_debug(self):
         self.config_fixture.config(debug=False)
@@ -177,11 +183,11 @@ class UnexpectedExceptionTestCase(ExceptionTestCase):
 
 
 class SecurityErrorTestCase(ExceptionTestCase):
-    """Tests whether security-related info is exposed to the API user."""
+    """Test whether security-related info is exposed to the API user."""
 
     def setUp(self):
         super(SecurityErrorTestCase, self).setUp()
-        self.config_fixture = self.useFixture(config_fixture.Config(cfg.CONF))
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
 
     def test_unauthorized_exposure(self):
         self.config_fixture.config(debug=False)
@@ -271,3 +277,33 @@ class SecurityErrorTestCase(ExceptionTestCase):
         e = exception.Forbidden(message=risky_info)
         self.assertValidJsonRendering(e)
         self.assertNotIn(risky_info, six.text_type(e))
+
+
+class TestSecurityErrorTranslation(unit.BaseTestCase):
+    """Test i18n for SecurityError exceptions."""
+
+    def setUp(self):
+        super(TestSecurityErrorTranslation, self).setUp()
+        self.config_fixture = self.useFixture(config_fixture.Config(CONF))
+        self.config_fixture.config(insecure_debug=False)
+        self.warning_log = self.useFixture(fixtures.FakeLogger(level=log.WARN))
+
+        exception._FATAL_EXCEPTION_FORMAT_ERRORS = False
+        self.addCleanup(
+            setattr, exception, '_FATAL_EXCEPTION_FORMAT_ERRORS', True)
+
+    class CustomSecurityError(exception.SecurityError):
+        message_format = _LE('We had a failure in the %(place)r')
+
+    class CustomError(exception.Error):
+        message_format = _LE('We had a failure in the %(place)r')
+
+    def test_nested_translation_of_SecurityErrors(self):
+        e = self.CustomSecurityError(place='code')
+        _LE('Admiral found this in the log: %s') % e
+        self.assertNotIn('programmer error', self.warning_log.output)
+
+    def test_that_regular_Errors_can_be_deep_copied(self):
+        e = self.CustomError(place='code')
+        _LE('Admiral found this in the log: %s') % e
+        self.assertNotIn('programmer error', self.warning_log.output)
