@@ -28,8 +28,9 @@ class BaseStyleCheck(unit.BaseTestCase):
         self.addCleanup(delattr, self, 'code_ex')
 
     def get_checker(self):
-        """Returns the checker to be used for tests in this class."""
-        raise NotImplemented('subclasses must provide a real implementation')
+        """Return the checker to be used for tests in this class."""
+        raise NotImplementedError('subclasses must provide '
+                                  'a real implementation')
 
     def get_fixture(self):
         return hacking_fixtures.HackingCode()
@@ -50,7 +51,7 @@ class BaseStyleCheck(unit.BaseTestCase):
 
     def assert_has_errors(self, code, expected_errors=None):
         actual_errors = [e[:3] for e in self.run_check(code)]
-        self.assertEqual(expected_errors or [], actual_errors)
+        self.assertItemsEqual(expected_errors or [], actual_errors)
 
 
 class TestCheckForMutableDefaultArgs(BaseStyleCheck):
