@@ -16,16 +16,16 @@ import subprocess
 import uuid
 
 import ldap.modlist
-from oslo_config import cfg
 from six.moves import range
 
+import keystone.conf
 from keystone import exception
 from keystone.identity.backends import ldap as identity_ldap
 from keystone.tests import unit
 from keystone.tests.unit import test_backend_ldap
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 def create_object(dn, attrs):
@@ -83,7 +83,7 @@ class LiveLDAPIdentity(test_backend_ldap.LDAPIdentity):
         # logic is different from the fake backend.
         user_api = identity_ldap.UserApi(CONF)
         self.assertTrue(user_api)
-        self.assertEqual(user_api.tree_dn, CONF.ldap.user_tree_dn)
+        self.assertEqual(CONF.ldap.user_tree_dn, user_api.tree_dn)
 
     def test_ldap_dereferencing(self):
         alt_users_ldif = {'objectclass': ['top', 'organizationalUnit'],
