@@ -15,18 +15,18 @@
 import copy
 import functools
 
-from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
 
 from keystone.common import sql
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _LI
 from keystone import token
 from keystone.token import provider
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -48,7 +48,7 @@ class TokenModel(sql.ModelBase, sql.DictBase):
 
 
 def _expiry_range_batched(session, upper_bound_func, batch_size):
-    """Returns the stop point of the next batch for expiration.
+    """Return the stop point of the next batch for expiration.
 
     Return the timestamp of the next token that is `batch_size` rows from
     being the oldest expired token.
@@ -77,7 +77,7 @@ def _expiry_range_batched(session, upper_bound_func, batch_size):
 
 
 def _expiry_range_all(session, upper_bound_func):
-    """Expires all tokens in one pass."""
+    """Expire all tokens in one pass."""
     yield upper_bound_func()
 
 
@@ -114,7 +114,7 @@ class Token(token.persistence.TokenDriverV8):
 
     def delete_tokens(self, user_id, tenant_id=None, trust_id=None,
                       consumer_id=None):
-        """Deletes all tokens in one session
+        """Delete all tokens in one session.
 
         The user_id will be ignored if the trust_id is specified. user_id
         will always be specified.
@@ -244,7 +244,7 @@ class Token(token.persistence.TokenDriverV8):
             return tokens
 
     def _expiry_range_strategy(self, dialect):
-        """Choose a token range expiration strategy
+        """Choose a token range expiration strategy.
 
         Based on the DB dialect, select an expiry range callable that is
         appropriate.
@@ -269,7 +269,12 @@ class Token(token.persistence.TokenDriverV8):
         return _expiry_range_all
 
     def flush_expired_tokens(self):
-        with sql.session_for_write() as session:
+        # The DBAPI itself is in a "never autocommit" mode,
+        # BEGIN is emitted automatically as soon as any work is done,
+        # COMMIT is emitted when SQLAlchemy invokes commit() on the
+        # underlying DBAPI connection. So SQLAlchemy is only simulating
+        # "begin" here in any case, it is in fact automatic by the DBAPI.
+        with sql.session_for_write() as session:  # Calls session.begin()
             dialect = session.bind.dialect.name
             expiry_range_func = self._expiry_range_strategy(dialect)
             query = session.query(TokenModel.expires)
@@ -279,8 +284,17 @@ class Token(token.persistence.TokenDriverV8):
                 delete_query = query.filter(TokenModel.expires <=
                                             expiry_time)
                 row_count = delete_query.delete(synchronize_session=False)
+                # Explicitly commit each batch so as to free up
+                # resources early. We do not actually need
+                # transactional semantics here.
+                session.commit()  # Emits connection.commit() on DBAPI
+                # Tells SQLAlchemy to "begin", e.g. hold a new connection
+                # open in a transaction
+                session.begin()
                 total_removed += row_count
                 LOG.debug('Removed %d total expired tokens', total_removed)
 
+            # When the "with: " block ends, the final "session.commit()"
+            # is emitted by enginefacade
             session.flush()
             LOG.info(_LI('Total expired tokens removed: %d'), total_removed)
