@@ -17,21 +17,22 @@ import uuid
 
 from cryptography import fernet
 import msgpack
-from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
+import six
 from six.moves import map
 from six.moves import urllib
 
 from keystone.auth import plugins as auth_plugins
+from keystone.common import fernet_utils as utils
 from keystone.common import utils as ks_utils
+import keystone.conf
 from keystone import exception
 from keystone.i18n import _, _LI
 from keystone.token import provider
-from keystone.token.providers.fernet import utils
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 # Fernet byte indexes as as computed by pypi/keyless_fernet and defined in
@@ -56,7 +57,11 @@ class TokenFormatter(object):
         ``encrypt(plaintext)`` and ``decrypt(ciphertext)``.
 
         """
-        keys = utils.load_keys()
+        fernet_utils = utils.FernetUtils(
+            CONF.fernet_tokens.key_repository,
+            CONF.fernet_tokens.max_active_keys
+        )
+        keys = fernet_utils.load_keys()
 
         if not keys:
             raise exception.KeysNotFound()
@@ -119,7 +124,7 @@ class TokenFormatter(object):
 
     @classmethod
     def creation_time(cls, fernet_token):
-        """Returns the creation time of a valid Fernet token.
+        """Return the creation time of a valid Fernet token.
 
         :type fernet_token: six.text_type
 
@@ -139,9 +144,9 @@ class TokenFormatter(object):
         timestamp_int = struct.unpack(">Q", timestamp_bytes)[0]
 
         # and with an integer, it's trivial to produce a datetime object
-        created_at = datetime.datetime.utcfromtimestamp(timestamp_int)
+        issued_at = datetime.datetime.utcfromtimestamp(timestamp_int)
 
-        return created_at
+        return issued_at
 
     def create_token(self, user_id, expires_at, audit_ids, methods=None,
                      domain_id=None, project_id=None, trust_id=None,
@@ -178,7 +183,7 @@ class TokenFormatter(object):
         return token
 
     def validate_token(self, token):
-        """Validates a Fernet token and returns the payload attributes.
+        """Validate a Fernet token and returns the payload attributes.
 
         :type token: six.text_type
 
@@ -201,13 +206,13 @@ class TokenFormatter(object):
 
         # rather than appearing in the payload, the creation time is encoded
         # into the token format itself
-        created_at = TokenFormatter.creation_time(token)
-        created_at = ks_utils.isotime(at=created_at, subsecond=True)
+        issued_at = TokenFormatter.creation_time(token)
+        issued_at = ks_utils.isotime(at=issued_at, subsecond=True)
         expires_at = timeutils.parse_isotime(expires_at)
         expires_at = ks_utils.isotime(at=expires_at, subsecond=True)
 
         return (user_id, methods, audit_ids, domain_id, project_id, trust_id,
-                federated_info, access_token_id, created_at, expires_at)
+                federated_info, access_token_id, issued_at, expires_at)
 
 
 class BasePayload(object):
@@ -534,7 +539,11 @@ class FederatedUnscopedPayload(BasePayload):
         (is_stored_as_bytes, idp_id) = payload[3]
         if is_stored_as_bytes:
             idp_id = cls.convert_uuid_bytes_to_hex(idp_id)
+        else:
+            idp_id = idp_id.decode('utf-8')
         protocol_id = payload[4]
+        if isinstance(protocol_id, six.binary_type):
+            protocol_id = protocol_id.decode('utf-8')
         expires_at_str = cls._convert_float_to_time_string(payload[5])
         audit_ids = list(map(provider.base64_encode, payload[6]))
         federated_info = dict(group_ids=group_ids, idp_id=idp_id,
